@@ -13,6 +13,8 @@ __PACKAGE__->attr(
     stash => sub { {} }
 );
 
+*all = \&fetch_hash_all;
+
 sub filter {
     my $self = shift;
     
@@ -247,6 +249,8 @@ sub fetch_multi {
     return $rows;
 }
 
+*one = \&fetch_hash_first;
+
 sub remove_end_filter {
     my $self = shift;
     
@@ -372,6 +376,12 @@ Statement handle of L<DBI>.
 L<DBIx::Custom::Result> inherits all methods from L<Object::Simple>
 and implements the following new ones.
 
+=head2 C<all>
+
+    my $rows = $result->all;
+
+This is alias for C<fetch_hash_all>.
+
 =head2 C<end_filter>
 
     $result = $result->end_filter(title  => 'to_something',
@@ -444,6 +454,12 @@ Filters.
 These each filters override the filters applied by C<apply_filter> of
 L<DBIx::Custom>.
 
+=head2 C<one>
+
+    my $row = $result->one;
+
+This is alias for C<fetch_hash_first>.
+
 =head2 C<remove_end_filter>
 
     $result->remove_end_filter;
